"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fieldMapping = fieldMapping;
exports.connMapping = connMapping;

var _graphqlRelay = require("graphql-relay");

function argMapping(args, nodeInterface) {
  return Object.keys(args).map(function (k) {
    var arg = args[k];
    var baseType = typeof arg.type === "function" && !arg.type.prototype ? arg.type() : arg.type;
    var type = baseType.GraphQuill ? baseType.GraphQLType || baseType.GraphQuill(nodeInterface).GraphQLType : baseType;
    var kv = {};
    kv[k] = {
      type: type,
      description: arg.description
    };
    return kv;
  }).reduce(function (obj, kv) {
    return Object.assign(obj, kv);
  }, {});
}

function fieldMapping(fields, nodeInterface, resolveOverride) {
  return Object.keys(fields).map(function (k) {
    var field = fields[k];
    var baseType = typeof field.type === "function" && !field.type.GraphQuill ? field.type() : field.type;
    var type = baseType.GraphQuill ? baseType.GraphQLType || baseType.GraphQuill(nodeInterface).GraphQLType : baseType;
    var kv = {};
    kv[k] = {
      type: type,
      description: field.description,
      args: field.args ? argMapping(field.args) : undefined,
      resolve: resolveOverride || function (obj, args, context, info) {
        return typeof obj[k] === "function" ? obj[k](args, context, info) : obj[k];
      }
    };
    return kv;
  }).reduce(function (obj, kv) {
    return Object.assign(obj, kv);
  }, {});
}

function connMapping(fields, nodeInterface, resolveOverride) {
  return Object.keys(fields).map(function (k) {
    var field = fields[k];
    var baseConnectedType = typeof field.connectedType === "function" && !field.connectedType.GraphQuill ? field.connectedType() : field.connectedType;
    var connectedType = baseConnectedType.GraphQLConnectionType || baseConnectedType.GraphQuill(nodeInterface).GraphQLConnectionType;
    var kv = {};
    kv[k] = {
      type: connectedType,
      description: field.description,
      args: Object.assign({}, !field.connectionDirection ? _graphqlRelay.connectionArgs : field.connectionDirection === "forward" ? _graphqlRelay.forwardConnectionArgs : _graphqlRelay.backwardConnectionArgs, field.args ? argMapping(field.args) : {}),
      resolve: function resolve(obj, args, context, info) {
        return (0, _graphqlRelay.connectionFromPromisedArray)(Promise.resolve(resolveOverride ? resolveOverride(obj, args, context, info) : typeof obj[k] === "function" ? obj[k](args, context, info) : obj[k]), args);
      }
    };
    return kv;
  }).reduce(function (obj, kv) {
    return Object.assign(obj, kv);
  }, {});
}