"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createSchema = createSchema;

var _graphql = require("graphql");

var _graphqlRelay = require("graphql-relay");

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function createSchema(types, rootQueriesAndConns) {
  var _nodeDefinitions = (0, _graphqlRelay.nodeDefinitions)(function (globalId) {
    var _fromGlobalId = (0, _graphqlRelay.fromGlobalId)(globalId);

    var type = _fromGlobalId.type;
    var id = _fromGlobalId.id;

    var matches = expandedTypes.filter(function (t) {
      return t.GraphQLType && t.GraphQLConnectionType && t.resolveById && t.GraphQLType.name === type;
    });
    return matches.length > 0 && matches[0].resolveById ? matches[0].resolveById(id) : Promise.reject(new Error("No matching Relay type for " + type));
  }, function (obj) {
    var matches = expandedTypes.filter(function (t) {
      return obj instanceof t;
    });
    return matches.length > 0 ? matches[0].GraphQLType : Promise.reject(new Error("No matching Relay type for object"));
  });

  var nodeInterface = _nodeDefinitions.nodeInterface;
  var nodeField = _nodeDefinitions.nodeField;

  var expandedTypes = types.map(function (possibleType) {
    if (possibleType.GraphQuill && !possibleType.GraphQLType) {
      return possibleType.GraphQuill(nodeInterface);
    } else if (possibleType.GraphQLType) {
      return possibleType;
    } else {
      throw new Error("Non-GraphQuill-annotated class");
    }
  });
  var expandedRootQueries = rootQueriesAndConns.map(function (possibleRootQuery) {
    if (possibleRootQuery.GraphQuill && !possibleRootQuery.GraphQLFields) {
      return possibleRootQuery.GraphQuill(nodeInterface);
    } else {
      throw new Error("Non-GraphQuill-annotated root query");
    }
  });
  var queryType = new _graphql.GraphQLObjectType({
    name: "Query",
    description: "Root Query: Graph Entry Point",
    fields: function fields() {
      return Object.assign.apply(Object, [{
        node: nodeField
      }].concat(_toConsumableArray(expandedRootQueries.map(function (exp) {
        return exp.GraphQLFields;
      }))));
    }
  });
  return new _graphql.GraphQLSchema({
    query: queryType
  });
}